""" Main menu scene. """

from pgzero.constants import keys
from pgzero.loaders import images
from ..actors.navigator import ActorNavigator
from ..actors.orientatedsurf import OrientatedSurfActor, SelectableSurfActor
from ..util import safepath
from .base import Scene, ChangeSceneEvent, WorldEvent, QuitEvent


class MenuScene(Scene):
    """ Main menu scene. """

    def __init__(self):
        super().__init__()
        self.actors.add_layer("buttons", level=10)
        self._init_bg()
        self._init_buttons()

    def _init_bg(self):
        self.actors.default.clear()
        self.actors.default.add(OrientatedSurfActor(
            images.load(safepath("splash/background"))))

    def _init_buttons(self):
        self._nav = ActorNavigator()
        menu = [
            self._build_button("new_game", action=self.change_to_new_game),
            self._build_button("resume_game", action=self.change_to_resume),
            self._build_button("credits", action=self.change_to_credits),
            self._build_button("quit", action=self.quit),
        ]
        for i, item in enumerate(menu):
            item.pos = (675, 275 + (i * 100))
            self.actors.add(item)
        self._nav.connect(
            menu, [keys.DOWN, keys.RIGHT], [keys.UP, keys.LEFT],
            wrap=True)
        self._nav.current.selected = True

    def _build_button(self, name, action):
        actor = SelectableSurfActor(
            images.load(safepath("splash/%s" % (name,))),
            images.load(safepath("splash/%s_selected" % (name,))))
        actor.action = action
        return actor

    def change_to_resume(self):
        from .level import GameLevelScene
        return [ChangeSceneEvent(GameLevelScene())]

    def change_to_new_game(self):
        from .level import GameLevelScene
        return [
            WorldEvent("reset"),
            ChangeSceneEvent(GameLevelScene())
        ]

    def change_to_credits(self):
        from .credits import CreditsScene
        return [ChangeSceneEvent(CreditsScene())]

    def quit(self):
        return [QuitEvent()]

    def on_key_down(self, key, mod, unicode):
        if key == keys.RETURN:
            current = self._nav.current
            if current is not None:
                return current.action()
        else:
            prev, current = self._nav.follow(key)
            if prev is not None:
                prev.selected = False
            if current is not None:
                current.selected = True
