"""Render a level and allow moving the scene"""

from pygame.constants import BLEND_RGBA_MULT
from pgzero.constants import keys
from pgzero.loaders import images
from .level import BaseLevelScene
from .base import MoveViewportEvent
from ..constants import TILE_SIZE
from ..util import safepath


class ViewLevelScene(BaseLevelScene):
    """ Level scene. """

    def enter(self, world):
        super().enter(world)
        # Mark starting position
        self._roach = images.load(safepath("roach/roach_1")).copy()
        self._roach.fill((255, 0, 0, 255), None, BLEND_RGBA_MULT)
        x = self._level.start_pos[0] * TILE_SIZE
        y = self._level.start_pos[1] * TILE_SIZE
        for layer in ['floor', 'tunnels']:
            self._surfaces[layer].blit(self._roach, (x, y))

    def on_key_down(self, key, mod, unicode):
        offset = None
        if key == keys.DOWN:
            offset = (0, TILE_SIZE)
        elif key == keys.UP:
            offset = (0, -TILE_SIZE)
        elif key == keys.LEFT:
            offset = (-TILE_SIZE, 0)
        elif key == keys.RIGHT:
            offset = (TILE_SIZE, 0)
        elif key == keys.S:
            if self._level_layer == 'floor':
                self._level_layer = 'tunnels'
            else:
                self._level_layer = 'floor'
            return
        if offset:
            return [MoveViewportEvent(offset)]
        return super(ViewLevelScene, self).on_key_down(key, mod, unicode)
